@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <math.h>
 
 #include <locale.h>
 
@@ -37,6 +38,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
+#include <gio/gdesktopappinfo.h>
 
 #include "gnome-settings-profile.h"
 #include "gsd-marshal.h"
@@ -468,42 +470,54 @@ dialog_show (GsdMediaKeysManager *manager)
         gdk_display_sync (gdk_screen_get_display (manager->priv->current_screen));
 }
 
+static void
+launch_app (GAppInfo *app_info,
+	    gint64    timestamp)
+{
+	GError *error = NULL;
+        GdkAppLaunchContext *launch_context;
+
+        /* setup the launch context so the startup notification is correct */
+        launch_context = gdk_display_get_app_launch_context (gdk_display_get_default ());
+        gdk_app_launch_context_set_timestamp (launch_context, timestamp);
+
+	if (!g_app_info_launch (app_info, NULL, G_APP_LAUNCH_CONTEXT (launch_context), &error)) {
+		g_warning ("Could not launch '%s': %s",
+			   g_app_info_get_commandline (app_info),
+			   error->message);
+		g_error_free (error);
+	}
+        g_object_unref (launch_context);
+}
+
 static void
 do_url_action (GsdMediaKeysManager *manager,
-               const char          *scheme)
+               const char          *scheme,
+               gint64               timestamp)
 {
         GAppInfo *app_info;
 
         app_info = g_app_info_get_default_for_uri_scheme (scheme);
         if (app_info != NULL) {
-                GError *error = NULL;
-
-                if (!g_app_info_launch (app_info, NULL, NULL, &error)) {
-                        g_warning ("Could not launch '%s': %s",
-                                   g_app_info_get_commandline (app_info),
-                                   error->message);
-                        g_error_free (error);
-                }
+                launch_app (app_info, timestamp);
+                g_object_unref (app_info);
         } else {
                 g_warning ("Could not find default application for '%s' scheme", scheme);
-        }
+	}
 }
 
 static void
-do_media_action (GsdMediaKeysManager *manager)
+do_media_action (GsdMediaKeysManager *manager,
+		 gint64               timestamp)
 {
         GAppInfo *app_info;
 
-        app_info = g_app_info_get_default_for_type ("audio/ogg", FALSE);
+        app_info = g_app_info_get_default_for_type ("audio/x-vorbis+ogg", FALSE);
         if (app_info != NULL) {
-                GError *error = NULL;
-
-                if (!g_app_info_launch (app_info, NULL, NULL, &error)) {
-                        g_warning ("Could not launch '%s': %s",
-                                   g_app_info_get_commandline (app_info),
-                                   error->message);
-                        g_error_free (error);
-                }
+                launch_app (app_info, timestamp);
+                g_object_unref (app_info);
+        } else {
+                g_warning ("Could not find default application for '%s' mime-type", "audio/x-vorbis+ogg");
         }
 }
 
@@ -584,6 +598,41 @@ do_eject_action (GsdMediaKeysManager *manager)
         g_object_unref (volume_monitor);
 }
 
+static void
+do_home_key_action (GsdMediaKeysManager *manager,
+		    gint64               timestamp)
+{
+	GFile *file;
+	GError *error = NULL;
+	char *uri;
+
+	file = g_file_new_for_path (g_get_home_dir ());
+	uri = g_file_get_uri (file);
+	g_object_unref (file);
+
+	if (gtk_show_uri (NULL, uri, timestamp, &error) == FALSE) {
+		g_warning ("Failed to launch '%s': %s", uri, error->message);
+		g_error_free (error);
+	}
+	g_free (uri);
+}
+
+static void
+do_execute_desktop (GsdMediaKeysManager *manager,
+		    const char          *desktop,
+		    gint64               timestamp)
+{
+        GDesktopAppInfo *app_info;
+
+        app_info = g_desktop_app_info_new (desktop);
+        if (app_info != NULL) {
+                launch_app (G_APP_INFO (app_info), timestamp);
+                g_object_unref (app_info);
+        } else {
+                g_warning ("Could not find application '%s'", desktop);
+	}
+}
+
 static void
 do_touchpad_osd_action (GsdMediaKeysManager *manager, gboolean state)
 {
@@ -622,8 +671,12 @@ update_dialog (GsdMediaKeysManager *manager,
                gboolean sound_changed,
                gboolean quiet)
 {
-        vol = (int) (100 * (double) vol / PA_VOLUME_NORM);
-        vol = CLAMP (vol, 0, 100);
+        if (!muted) {
+                vol = (int) (100 * (double) vol / PA_VOLUME_NORM);
+                vol = CLAMP (vol, 0, 100);
+        } else {
+                vol = 0.0;
+        }
 
         dialog_init (manager);
         gsd_media_keys_window_set_volume_muted (GSD_MEDIA_KEYS_WINDOW (manager->priv->dialog),
@@ -1345,7 +1398,7 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
 
         /* call into the power plugin */
         g_dbus_proxy_call (manager->priv->power_keyboard_proxy,
-                           type == SCREEN_BRIGHTNESS_UP_KEY ? "StepUp" : "StepDown",
+                           type == KEYBOARD_BRIGHTNESS_UP_KEY ? "StepUp" : "StepDown",
                            NULL,
                            G_DBUS_CALL_FLAGS_NONE,
                            -1,
@@ -1354,51 +1407,12 @@ do_keyboard_brightness_action (GsdMediaKeysManager *manager,
                            manager);
 }
 
-static void
-exec_battery_info (GsdMediaKeysManager *manager)
-{
-        gboolean ret;
-        GError *error = NULL;
-        GAppInfo *app_info;
-        GdkAppLaunchContext *launch_context;
-
-        /* setup the launch context so the startup notification is correct */
-        launch_context = gdk_display_get_app_launch_context (gdk_display_get_default ());
-        app_info = g_app_info_create_from_commandline (BINDIR "/gnome-power-statistics",
-                                                       "gnome-power-statistics",
-                                                       G_APP_INFO_CREATE_SUPPORTS_STARTUP_NOTIFICATION,
-                                                       &error);
-        if (app_info == NULL) {
-                g_warning ("failed to create application info: %s",
-                           error->message);
-                g_error_free (error);
-                goto out;
-        }
-
-        /* launch gnome-control-center */
-        ret = g_app_info_launch (app_info,
-                                 NULL,
-                                 G_APP_LAUNCH_CONTEXT (launch_context),
-                                 &error);
-        if (!ret) {
-                g_warning ("failed to launch gnome-power-statistics: %s",
-                           error->message);
-                g_error_free (error);
-                goto out;
-        }
-out:
-        g_object_unref (launch_context);
-        if (app_info != NULL)
-                g_object_unref (app_info);
-}
-
 static gboolean
 do_action (GsdMediaKeysManager *manager,
            MediaKeyType         type,
            gint64               timestamp)
 {
         char *cmd;
-        char *path;
 
         g_debug ("Launching action for key type '%d'", type);
 
@@ -1441,47 +1455,34 @@ do_action (GsdMediaKeysManager *manager,
                 do_eject_action (manager);
                 break;
         case HOME_KEY:
-                path = g_shell_quote (g_get_home_dir ());
-                cmd = g_strconcat ("nautilus --no-desktop ", path, NULL);
-                g_free (path);
-                execute (manager, cmd, FALSE, FALSE);
-                g_free (cmd);
+                do_home_key_action (manager, timestamp);
                 break;
         case SEARCH_KEY:
                 cmd = NULL;
-                if ((cmd = g_find_program_in_path ("beagle-search"))) {
-                        execute (manager, "beagle-search", FALSE, FALSE);
-                } else if ((cmd = g_find_program_in_path ("tracker-search-tool"))) {
-                        execute (manager, "tracker-search-tool", FALSE, FALSE);
-                } else {
-                        execute (manager, "gnome-search-tool", FALSE, FALSE);
-                }
+                if ((cmd = g_find_program_in_path ("tracker-search-tool")))
+                        do_execute_desktop (manager, "tracker-needle.desktop", timestamp);
+                else
+                        do_execute_desktop (manager, "gnome-search-tool.desktop", timestamp);
                 g_free (cmd);
                 break;
         case EMAIL_KEY:
-                do_url_action (manager, "mailto");
+                do_url_action (manager, "mailto", timestamp);
                 break;
         case SCREENSAVER_KEY:
         case SCREENSAVER2_KEY:
-                if ((cmd = g_find_program_in_path ("gnome-screensaver-command"))) {
-                        execute (manager, "gnome-screensaver-command --lock", FALSE, FALSE);
-                } else {
-                        execute (manager, "xscreensaver-command -lock", FALSE, FALSE);
-                }
-
-                g_free (cmd);
+                execute (manager, "gnome-screensaver-command --lock", FALSE, FALSE);
                 break;
         case HELP_KEY:
-                do_url_action (manager, "ghelp");
+                do_url_action (manager, "ghelp", timestamp);
                 break;
         case WWW_KEY:
-                do_url_action (manager, "http");
+                do_url_action (manager, "http", timestamp);
                 break;
         case MEDIA_KEY:
-                do_media_action (manager);
+                do_media_action (manager, timestamp);
                 break;
         case CALCULATOR_KEY:
-                execute (manager, "gcalctool", FALSE, FALSE);
+                do_execute_desktop (manager, "gcalctool.desktop", timestamp);
                 break;
         case PLAY_KEY:
                 return do_multimedia_player_action (manager, "Play");
@@ -1550,7 +1551,7 @@ do_action (GsdMediaKeysManager *manager,
                 do_keyboard_brightness_action (manager, type);
                 break;
         case BATTERY_KEY:
-                exec_battery_info (manager);
+                do_execute_desktop (manager, "gnome-power-statistics.desktop", timestamp);
                 break;
         case HANDLED_KEYS:
                 g_assert_not_reached ();
